import 'dart:ui';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:line_awesome_flutter/line_awesome_flutter.dart';
import 'package:nui_animation/nui_animation.dart';
import 'package:nui_core/nui_core.dart';
import 'package:nui_core/ui/component/listview/nui_header_gridview.dart';
import 'package:nui_media/model/nui_media_file.dart';
import 'package:nui_media/model/nui_media_list_repo.dart';
import 'package:nui_media/nui_media.dart';
import 'package:nui_media/ui/nui_media_preview.dart';

class NUIMediaSelectionGridBloc{
  NUIMediaSelectionGridBloc({this.type: NUIMediaGalleryOption.BOTH, this.postFilesLoadCallback});
  final NUIMediaGalleryOption type;
  var pageBloc = NUIBlocData<NUIMediaSelectionPageData>();
  var pageBloc2 = NUIBlocData<NUIMediaSelectionPageData>();
  Function postFilesLoadCallback;
  List<NUIMediaFile> curFiles = List();

  NUIListViewBloc<NUIMediaFile, NUIMediaFile> filesBloc;
  List<NUIMediaFile> additionalFiles = List();

  NUIListViewBloc<NUIMediaFile, NUIMediaFile> getFilesBloc(){
    filesBloc = filesBloc ?? NUIListViewBloc<NUIMediaFile, NUIMediaFile>(callbackMethod: (current, isFirst, append, cursor) async{
      curFiles.clear();

      final medias = NUIMedia.get().getFromGallery(option: type, limit: 200);
      curFiles.addAll(await medias);

      logNUI("NUIMediaSelectionPreview", "Total selections available is ${curFiles.length}");
      if(postFilesLoadCallback != null) {
        NUIAsync.delay(Duration(milliseconds: 250), () {
          postFilesLoadCallback();
        });
      }
      return ListResult<NUIMediaFile, NUIMediaFile>(data: curFiles, success: true);
    });
    return filesBloc;
  }

  Future<List<NUIMediaFile>> getSelectedFiles(NUIListController controller, List<NUIMediaFile> selectedPaths) async{
    var files = List<NUIMediaFile>();
    for(NUIMediaFile f in curFiles){
      if(!isNullOrEmpty(selectedPaths)){
        NUIMediaFile pathMatch;
        for(NUIMediaFile path in selectedPaths){
          if(match(path.path, f.path)){
            f.sequence = path.sequence;
            f.croppedFileName = path.croppedFileName;
            f.cropped = path.cropped;
            files.add(f);
            pathMatch = path;
            break;
          }
        }
        if(pathMatch != null) selectedPaths.remove(pathMatch);
      }
      else{
        break;
      }
    }
    if(!selectedPaths.isNullOrEmpty()){
      //Some other files that are not found previously
      for(NUIMediaFile file in selectedPaths){
        controller.insertItemAt(0, file);
      }
      controller.refreshList();
      additionalFiles.clear();
      additionalFiles.addAll(selectedPaths);
      files.addAll(selectedPaths);
    }
    files.sort((first, second){
      return (first.sequence?? 0).compareTo((second.sequence ?? 0));
    });
    return files;
  }
}

class NUIMediaSelectionPageData{
  NUIMediaSelectionPageData({this.position: 0, this.files});
  int position;
  List<NUIMediaFile> files;
}

class NUIMediaSelectionPreview extends StatefulWidget {
  NUIMediaSelectionPreview({this.maximumSelection, this.minimumSelection, this.enableCrop: true, this.cropRatio: 1.0, this.selections, this.type: NUIMediaGalleryOption.BOTH, this.accentColor, this.accentColorBright});
  final int maximumSelection;
  final int minimumSelection;
  final NUIMediaGalleryOption type;
  final Color accentColor;
  final Color accentColorBright;
  final bool enableCrop;
  final double cropRatio;
  final List<NUIMediaFile> selections;
  @override
  _NUIMediaSelectionPreviewState createState() => _NUIMediaSelectionPreviewState();
}

class _NUIMediaSelectionPreviewState extends State<NUIMediaSelectionPreview> {
  NUIMediaSelectionGridBloc bloc;
  List<NUIMediaFile> _selectedFiles = List();
  NUIListController listController = NUIListController();
  NUIPageViewController pageController = NUIPageViewController();
  bool _singleSelection;

  @override
  void initState(){
    super.initState();
    bloc = NUIMediaSelectionGridBloc(type: widget.type ?? NUIMediaGalleryOption.BOTH, postFilesLoadCallback: (){
      _fillInPreviousSelection();
    });
    _singleSelection = widget.maximumSelection == 1;
  }

  void _fillInPreviousSelection() async{
    logNUI("NUIMediaSelectionPreview", "Total selected files filled in : ${widget.selections?.length}");
    if(isNullOrEmpty(widget.selections)) return;

    final files = await bloc.getSelectedFiles(listController, widget.selections);
    _selectedFiles.addAll(files);
    logNUI("NUIMediaSelectionPreview", "Total selected files found : ${_selectedFiles.length}");
    _updateSequence();
    bloc.pageBloc.notify(NUIMediaSelectionPageData(position: _selectedFiles.length - 1, files: _selectedFiles));
    bloc.pageBloc2.notify(NUIMediaSelectionPageData(position: _selectedFiles.length - 1, files: _selectedFiles));
  }

  void _updateSequence(){
    listController.updateCheckedMap((map) {
      map.clear();
      int position = 1;
      for(NUIMediaFile file in _selectedFiles){
        file.sequence = position - 1;
        map[file.path] = position;
        position++;
      }
    });
    bloc.pageBloc.notify(NUIMediaSelectionPageData(position: _selectedFiles.length - 1, files: _selectedFiles));
    bloc.pageBloc2.notify(NUIMediaSelectionPageData(position: _selectedFiles.length - 1, files: _selectedFiles));
//    pageController.changePage(_selectedFiles.length-1);
  }

  void _confirm(BuildContext context){
    if(widget.minimumSelection != null && _selectedFiles.length < widget.minimumSelection){
      toast("Select at least ${widget.minimumSelection} from your gallery", context: context, textStyle: getFont(context, size: 14, singleColor: NUIColors.NUIWhite));
      return;
    }

    Navigator.pop(context, _selectedFiles);
  }

  void _back(){
    Navigator.pop(context, _selectedFiles);
  }

  void _addSingleFile(NUIMediaFile file){
    _selectedFiles.add(file);
    _updateSequence();
  }

  void _addCroppedFile(NUIMediaFile croppedFile){
    logNUI("NUIMediaSelectionPreview", "Adding cropped file : ${croppedFile.croppedFileName}");
    final targetCroppedName = croppedFile.croppedFileName;
    for(NUIMediaFile file in _selectedFiles){
      if(match(file.path, croppedFile.originalPath)){
        logNUI("NUIMediaSelectionPreview", "Removing selection for original cropped file");
        _selectedFiles.remove(file);
        break;
      }
    }
    listController.updateConfigMap((map) {
      map["newCroppedFile"] = targetCroppedName;
    });
    _updateSequence();
    setState(() {

    });
  }

  void _processClickAction(dynamic data, int position, String action){
    if(match(action, NUIListActions.ADD)){
      if(_singleSelection == true && _selectedFiles.isEmpty) {
        logNUI("NUIMediaSelectionPreview", "Adding file for single selection");
        _selectedFiles.add(data);
        _updateSequence();
      }
      else if(_singleSelection != true && widget.maximumSelection != null && _selectedFiles.length < widget.maximumSelection){
        logNUI("NUIMediaSelectionPreview", "Adding file with maximum selection of ${widget.maximumSelection}");
        _selectedFiles.add(data);
        _updateSequence();
      }
      else if(_singleSelection != true && widget.maximumSelection == null){
        logNUI("NUIMediaSelectionPreview", "Adding file without maximum selection");
        _selectedFiles.add(data);
        _updateSequence();
      }
    }
    else if(match(action, NUIListActions.REMOVE)){
      logNUI("NUIMediaSelectionPreview", "Removing file");
      _selectedFiles.remove(data);
      _updateSequence();
    }
  }

  void _removeFileByPath(String path){
    bool found = false;
    for(NUIMediaFile file in _selectedFiles){
      if(match(path, file.path)){
        _selectedFiles.remove(file);
        break;
      }
    }
    if(found) _updateSequence();
  }

  @override
  Widget build(BuildContext context) {
//    final totalHeight = screenHeight(context);
    return Scaffold(
      backgroundColor: NUIColors.NUITransparent,
      body: NUIScreen(
        screenCode: "NUIMediaSelectionPreview",
        overlayStyle: NUIOverlayStyle(darkStatusBar: false),
        content: (context) => SafeArea(
          top: true,
          child: Container(
            width: double.infinity,
            height: double.infinity,
            padding: EdgeInsets.only(top: 10),
            child: NUIColumn(
              mainAxisSize: MainAxisSize.max,
              divider: Divider(height: 10, color: NUIColors.NUITransparent),
              children: [
                Expanded(
                    flex: 55,
                    child: NUIBlocStream<NUIMediaSelectionPageData>(
                      bloc: bloc.pageBloc,
                      stream: (context, status, data){
                        if(status == BlocStatus.COMPLETED){
                          return AnimatedOpacity(
                            duration: Duration(milliseconds: 300),
                            opacity: data.files.isNotEmpty ? 1 : 0,
                            child: NUIMediaPagerPreview(
                              files: data.files,
                              croppedListener: (croppedFile){
                                _addCroppedFile(croppedFile);
                              },
                              controller: pageController,
                              enableCrop: widget.enableCrop ?? true,
                              cropRatio: widget.cropRatio ?? 1.0,
                              enableFullScreen: false,
                              initialPage: data.position,
                            ),
                          );
                        }
                        return Container();
                      }
                    )
                ),
                Expanded(
                    flex: 45,
                    child: Stack(
                      children: [
                        NUIShaderMask(
                          shaderType: NUIShaderMaskType.TOP_FADE,
                          shadeOffset: 0.15,
                          child: NUIHeaderGrid<NUIMediaFile, NUIMediaFile>(
                            padding: EdgeInsets.only(left: 12.5, right: 12.5, top: 50, bottom: 12.5),
                            bloc: bloc.getFilesBloc(),
                            controller: listController,
                            initialList: [],
                            enablePagination: false,
                            skeletalCount: 6,
                            skeletalInflater: NUILoadingListItem(inflater: (context) => NUIMediaListItemImageGridSkeletal()),
                            configMap: {
                              "singleSelection" : _singleSelection,
                              "accentColor": widget.accentColor,
                            },
                            repo: NUIMediaListItemRepo(),
                            gridDelegate: SliverGridDelegateWithFixedCrossAxisCount(mainAxisSpacing: 2, crossAxisSpacing: 2, childAspectRatio: 1.0, crossAxisCount: 4),
                            clickListener: (data, position, action){
                              _processClickAction(data, position, action);
                            },
                          ),
                        ),
                        Positioned(
                          right: 15,
                          child: NUIRow(
                            mainAxisSize: MainAxisSize.min,
                            crossAxisAlignment: CrossAxisAlignment.center,
                            divider: VerticalDivider(width: 10, color: NUIColors.NUITransparent,),
                            children: [
                              NUIBlocStream<NUIMediaSelectionPageData>(
                                bloc: bloc.pageBloc2,
                                stream: (context, state, data){
                                  if(state != BlocStatus.COMPLETED || data.files.length <= 0 || widget.maximumSelection == 1) return Container();
                                  return Container(
                                    alignment: Alignment.center,
                                    padding: EdgeInsets.only(left: 10, right: 10, top: 3, bottom: 3),
//                                    margin: EdgeInsets.only(top: 5),
                                    decoration: NUIOutlineDecoration(
                                        outlineColor: NUIColors.NUIWhite.withOpacity(0.0),
                                        gradient: NUILinearGradient(
                                            colors: [
                                              NUIColors.NUIWhite.withOpacity(0.5),
                                              NUIColors.NUIWhite.withOpacity(0.3),
                                            ],
                                            orientation: GradientOrientation.TL_BR
                                        ),
                                        thickness: 1.5,
                                        borderRadius: BorderRadius.circular(15)
                                    ),
                                    child: Text(
                                      widget.maximumSelection == null ? "${_selectedFiles.length} selected" : "${_selectedFiles.length}/${widget.maximumSelection} selected",
                                      style: getFont(context, size: 10, singleColor: NUIColors.NUIWhite, isBold: true),
                                    ),
                                  );
                                },
                              ),
                              NUIBouncingAnimation(
                                child: Container(
                                  height: 40,
                                  alignment: Alignment.center,
                                  padding: EdgeInsets.only(left: 15, right: 15),
                                  decoration: NUIBoxDecoration(
                                      borderRadiusCircular: 20,
                                      gradient: NUILinearGradient(
                                          colors: [
                                            widget.accentColor ?? NUIColors.NUIAccent,
                                            widget.accentColorBright ?? widget.accentColor ?? Color(0xFFF68585)
                                          ],
                                          orientation: GradientOrientation.TL_BR
                                      ),
                                      shadowOffset: Offset(1,1),
                                      shadowRadius: 4,
                                      shadowColor: NUIColors.NUIAccent.withOpacity(0.4)
                                  ),
                                    child: Icon(LineAwesomeIcons.check, color: NUIColors.NUIWhite, size: 24)
                                ),
                                onTap: (){
                                  _confirm(context);
                                },
                                scaleFactor: 0.5,
                              ),
                            ],
                          ),
                        ),
                        Positioned(
                          left: 15,
                          child: NUIBouncingAnimation(
                            child: Container(
                              height: 40,
                              width: 40,
                              alignment: Alignment.center,
                              decoration: NUIOutlineDecoration(
                                  outlineColor: NUIColors.NUIWhite.withOpacity(0.6),
                                  gradient: NUILinearGradient(
                                      colors: [
                                        NUIColors.NUIWhite.withOpacity(0.5),
                                        NUIColors.NUIWhite.withOpacity(0.3),
                                      ],
                                      orientation: GradientOrientation.TL_BR
                                  ),
                                  thickness: 1.5,
                                  borderRadius: BorderRadius.circular(20)
                              ),
                              child: Icon(LineAwesomeIcons.arrow_left, color: NUIColors.NUIWhite, size: 24)
                            ),
                            onTap: (){
                              _back();
                            },
                            scaleFactor: 0.5,
                          ),
                        ),
                      ],
                    )
                )
              ],
            ),
          ),
        ),
      ),
    );
  }
}


class NUIMediaListItemGridSkeletalModel{}
class NUIMediaListItemImageGridSkeletal extends NUIListStatefulWidget {
  @override
  _NUIMediaListItemImageGridSkeletalState createState() => _NUIMediaListItemImageGridSkeletalState();
}

class _NUIMediaListItemImageGridSkeletalState extends State<NUIMediaListItemImageGridSkeletal> {
  @override
  Widget build(BuildContext context) {
    return NUISkeletonContainer(
      baseColor: NUIColors.NUIWhite.withOpacity(0.2),
      highlightColor: NUIColors.NUIWhite.withOpacity(0.3),
      child: Container(
          width: double.infinity,
          height: double.infinity,
          child: Container(
          margin: EdgeInsets.all(2.5),
            decoration: NUIBoxDecoration(
              shadowRadius: 4,
              borderRadiusCircular: 15,
              shadowColor: NUIColors.NUIBlack.withOpacity(0.1),
              shadowOffset: Offset(1,1),
            ),
            child: NUISkeletonView(
              height: double.infinity,
              width: double.infinity,
              radius: BorderRadius.circular(15),
            ),
          )
      ),
    );
  }
}


class NUIMediaListItemImageGrid extends NUIListStatefulWidget {
  NUIMediaListItemImageGrid({this.mediaFile, this.size, NUIListActionController controller}): super(controller: controller);
  final NUIMediaFile mediaFile;
  final double size;
  @override
  _NUIListItemImageGridState createState() => _NUIListItemImageGridState();
}

class _NUIListItemImageGridState extends NUIListState<NUIMediaListItemImageGrid> {
  @override
  Widget build(BuildContext context) {
    final selectedPosition = getCheckedData(widget.mediaFile.path) as int;
    final newCroppedFile = getConfigData("newCroppedFile");
    final selected = selectedPosition != null;

    final size = screenWidth(context)/4;
    final singleSelection = getConfigData("singleSelection") == true;
    final accentColor = getConfigData("accentColor") as Color;

    if(!isNullOrEmpty(newCroppedFile) && contains(widget.mediaFile.path, newCroppedFile)){
      NUIAsync.delay(Duration(milliseconds: 500), () {
        getController().updateConfigData((map) {
          map.remove("newCroppedFile");
        });
        getController().onItemClick(widget.mediaFile, selected ? NUIListActions.REMOVE : NUIListActions.ADD);
      });
    }
    return NUIBouncingAnimation(
      onTap: (){
        getController().onItemClick(widget.mediaFile, selected ? NUIListActions.REMOVE : NUIListActions.ADD);
      },
      scaleFactor: 0.5,
      child: Container(
        width: double.infinity,
        height: double.infinity,
        child: Container(
          margin: EdgeInsets.all(2.5),
          decoration: NUIBoxDecoration(
            shadowRadius: 4,
            borderRadiusCircular: 15,
            shadowColor: NUIColors.NUIBlack.withOpacity(0.1),
            shadowOffset: Offset(1,1),
          ),
          child: NUIClipView(
            borderRadius: BorderRadius.circular(15),
            child: Stack(
              children: [
                widget.mediaFile.thumbData == null
                    ? loadImageLocal(widget.mediaFile.path, size, size)
                    : NUIFutureWidget(
                    futureValue: widget.mediaFile.thumbData,
                    callback: (state, data, message){
                      if(state == FutureState.SUCCESS) return loadImageLocal(widget.mediaFile.path, size, size, thumbdata: data);
                      return SizedBox(width: size, height: size);
                    }),
                Visibility(
                  visible: selected,
                  child: Container(
                    width: double.infinity,
                    height: double.infinity,
                    color: accentColor?.withOpacity(0.3) ?? NUIColors.NUIAccent.withOpacity(0.3),
                  )
                ),
                Visibility(
                  visible: selected,
                  child: Positioned(
                    top: 5,
                    right: 5,
                    child: NUIClipView(
                      borderRadius: BorderRadius.circular(15),
                      child: BackdropFilter(
                        filter: ImageFilter.blur(sigmaX: 5.0, sigmaY: 5.0),
                        child: Container(
                            height: 30,
                            width: 30,
                            alignment: Alignment.center,
                            decoration: NUIOutlineDecoration(
                              outlineColor: NUIColors.NUIWhite.withOpacity(0.6),
                              gradient: NUILinearGradient(
                                colors: [
                                  NUIColors.NUIBlack.withOpacity(0.2),
                                  NUIColors.NUIBlack.withOpacity(0.1),
                                ],
                                orientation: GradientOrientation.TL_BR
                              ),
                              thickness: 1.5,
                              borderRadius: BorderRadius.circular(15)
                            ),
                            child: !singleSelection ? Text(selectedPosition?.toString() ?? "0", style: getFont(context, size: 12, singleColor: NUIColors.NUIWhite, isBold: true)) :
                                Icon(LineAwesomeIcons.check, size: 20, color: NUIColors.NUIWhite,)
                        ),
                      ),
                    ),
                  ),
                ),
                Visibility(
                  visible: widget.mediaFile.thumbData != null,
                  child: Positioned(
                    top: 5,
                    left: 5,
                    child: NUIClipView(
                      borderRadius: BorderRadius.circular(15),
                      child: BackdropFilter(
                        filter: ImageFilter.blur(sigmaX: 5.0, sigmaY: 5.0),
                        child: Container(
                            height: 30,
                            width: 30,
                            alignment: Alignment.center,
                            decoration: NUIBoxDecoration(
                                color: NUIColors.NUIBlack.withOpacity(0.1)
                            ),
                            child: Icon(LineAwesomeIcons.play, size: 20, color: NUIColors.NUIWhite)
                        ),
                      ),
                    ),
                  ),
                ),
              ],
            ),
          ),
        ),
      ),
    );
  }
}


